import { NextRequest, NextResponse } from 'next/server';
import Stripe from 'stripe';

const stripe = new Stripe(process.env.STRIPE_SECRET_KEY || '');

export async function POST(request: NextRequest): Promise<NextResponse> {
  const { amount, payment_intent_id, metadata } = await request.json();

  if (payment_intent_id) {
    try {
      // If a payment_intent_id is passed, retrieve the paymentIntent
      const current_intent = await stripe.paymentIntents.retrieve(
        payment_intent_id,
      );
      // If a paymentIntent is retrieved update its amount
      if (current_intent) {
        const updated_intent = await stripe.paymentIntents.update(
          payment_intent_id,
          {
            amount: amount,
            metadata: metadata || {},
          },
        );

        return NextResponse.json(updated_intent, { status: 200 });
      }
    } catch (e) {
      // Catch any error and return a status 500
      if ((e as any).code !== 'resource_missing') {
        const errorMessage =
          e instanceof Error ? e.message : 'Internal server error';
        return NextResponse.json(
          { statusCode: 500, message: errorMessage },
          { status: 500 },
        );
      }
    }
  }

  try {
    // Create PaymentIntent
    const params: Stripe.PaymentIntentCreateParams = {
      amount: amount,
      currency: 'usd',
      payment_method_types: ['card'],
      automatic_payment_methods: {
        enabled: false,
      },
      metadata: metadata || {},
    };
    const payment_intent = await stripe.paymentIntents.create(params);
    //Return the payment_intent object
    return NextResponse.json(payment_intent, { status: 200 });
  } catch (err) {
    const errorMessage =
      err instanceof Error ? err.message : 'Internal server error';
    return NextResponse.json(
      { statusCode: 500, message: errorMessage },
      { status: 500 },
    );
  }
}
